"use strict";
const serenity_1 = require("../serenity");
const domain_1 = require("../serenity/domain");
const controlflow_1 = require("./controlflow");
const model_1 = require("./model");
// todo: [ ] tags: https://medium.com/@andrew_levine/tagging-tests-w-protractor-and-mocha-20b20bc10322#.cngxajsbf
// todo: [ ] initialise serenity
// todo: [ ] Update the tutorials to use See
// commits:
// - verification functions
//  - update the tuts to reflect that
/**
 * Execute the Runner's test cases through Mocha.
 *
 * @param {any} runner The current Protractor Runner.
 * @param {Array} specs Array of Directory Path Strings.
 * @return {Promise} Promise resolved with the test results
 */
exports.run = function run(runner, specs) {
    let Mocha = require('mocha'), mocha = new Mocha(runner.getConfig().mochaOpts);
    function protractorEvent(scenarioResult) {
        return {
            name: model_1.nameOf(scenarioResult),
            category: model_1.categoryOf(scenarioResult),
        };
    }
    return new Promise((resolve, reject) => {
        // Mocha doesn't set up the ui until the pre-require event, so
        // wait until then to load mocha-webdriver adapters as well.
        mocha.suite.on('pre-require', function () {
            try {
                let g = global;
                g.after = controlflow_1.wrapped(g.after);
                g.afterEach = controlflow_1.wrapped(g.afterEach);
                g.before = controlflow_1.wrapped(g.before);
                g.beforeEach = controlflow_1.wrapped(g.beforeEach);
                g.it = controlflow_1.wrapped(g.it);
                g.it.only = controlflow_1.wrapped(g.iit);
                g.it.skip = controlflow_1.wrapped(g.xit);
            }
            catch (err) {
                reject(err);
            }
        });
        mocha.loadFiles();
        runner.runTestPreparer().then(function () {
            specs.forEach(file => mocha.addFile(file));
            let results = [];
            let mochaRunner = mocha.run(function (failures) {
                try {
                    let completed = !!runner.getConfig().onComplete
                        ? Promise.resolve(runner.getConfig().onComplete())
                        : Promise.resolve();
                    completed.then(() => resolve({
                        failedCount: failures,
                        specResults: results,
                    }));
                }
                catch (err) {
                    reject(err);
                }
            });
            mochaRunner.on('test', function (scenario) {
                serenity_1.Serenity.notify(new domain_1.SceneStarts(new model_1.MochaScene(scenario)));
            });
            mochaRunner.on('test end', function (scenario) {
                if (model_1.finalStateOf(scenario) === domain_1.Result.PENDING) {
                    serenity_1.Serenity.notify(new domain_1.SceneStarts(new model_1.MochaScene(scenario)));
                }
                serenity_1.Serenity.notify(new domain_1.SceneFinished(new domain_1.Outcome(new model_1.MochaScene(scenario), model_1.finalStateOf(scenario), scenario.err)));
            });
            mochaRunner.on('pass', function (scenario) {
                runner.emit('testPass', protractorEvent(scenario));
                results.push({
                    description: scenario.title,
                    assertions: [{
                            passed: true,
                        }],
                    duration: scenario.duration,
                });
            });
            mochaRunner.on('fail', function (scenario) {
                runner.emit('testFail', protractorEvent(scenario));
                results.push({
                    description: scenario.title,
                    assertions: [{
                            passed: false,
                            errorMsg: scenario.err.message,
                            stackTrace: scenario.err.stack,
                        }],
                    duration: scenario.duration,
                });
            });
        }, reject);
    });
};
//# sourceMappingURL=index.js.map