"use strict";
const async_1 = require("../../../serenity/recording/async");
class BrowseTheWeb {
    constructor(browser) {
        this.browser = browser;
    }
    /**
     *
     * Instantiates the Ability to BrowseTheWeb, allowing the Actor to interact with a Web UI
     *
     * @param browser
     * @return {BrowseTheWeb}
     */
    static using(browser) {
        return new BrowseTheWeb(browser);
    }
    /**
     * Used to access the Actor's ability to BrowseTheWeb from within the Interaction classes, such as Click or Enter
     *
     * @param actor
     * @return {BrowseTheWeb}
     */
    static as(actor) {
        return actor.abilityTo(BrowseTheWeb);
    }
    locate(target) {
        return target.resolveUsing(this.browser.element);
    }
    locateAll(target) {
        return target.resolveAllUsing(this.browser.element); // see: https://github.com/angular/protractor/issues/3350
    }
    takeScreenshot() {
        return async_1.defer(() => this.browser.takeScreenshot());
    }
    get(destination, timeout) {
        return this.browser.get(destination, timeout);
    }
    getTitle() {
        return this.browser.getTitle();
    }
    actions() {
        return this.browser.actions();
    }
    manage() {
        return this.browser.driver.manage();
    }
    sleep(millis) {
        return async_1.defer(() => this.browser.sleep(millis));
    }
    wait(condition, timeout, message) {
        return this.browser.wait(condition, timeout, message);
    }
}
exports.BrowseTheWeb = BrowseTheWeb;
//# sourceMappingURL=browse_the_web.js.map