"use strict";
const browse_the_web_1 = require("../abilities/browse_the_web");
const protractor_1 = require("protractor");
class Duration {
    constructor(milliseconds) {
        this.milliseconds = milliseconds;
        this.toMillis = () => this.milliseconds;
        this.toString = () => `${this.milliseconds}ms`;
    }
}
Duration.ofMillis = (milliseconds) => new Duration(milliseconds);
Duration.ofSeconds = (seconds) => Duration.ofMillis(seconds * 1000);
exports.Duration = Duration;
class Wait {
    static until(somethingToWaitFor, condition) {
        return new ActiveWait().until(somethingToWaitFor, condition);
    }
}
Wait.for = (duration) => new PassiveWait(duration);
Wait.upTo = (timeout) => new ActiveWait(timeout);
exports.Wait = Wait;
class ActiveWait {
    constructor(timeout = ActiveWait.Default_Timeout) {
        this.timeout = timeout;
    }
    until(somethingToWaitFor, condition) {
        return condition(somethingToWaitFor, this.timeout);
    }
}
ActiveWait.Default_Timeout = Duration.ofSeconds(5);
exports.ActiveWait = ActiveWait;
class Is {
    static aTargetThat(condition) {
        return (target, timeout) => new WaitUntil(target, condition, timeout);
    }
}
Is.visible = () => Is.aTargetThat(new IsVisible());
Is.invisible = () => Is.aTargetThat(new IsInvisible());
Is.present = () => Is.aTargetThat(new IsPresent());
Is.absent = () => Is.aTargetThat(new Absent());
Is.selected = () => Is.aTargetThat(new IsSelected());
Is.clickable = () => Is.aTargetThat(new IsClickable());
exports.Is = Is;
// package-protected
class PassiveWait {
    constructor(duration) {
        this.duration = duration;
        this.performAs = (actor) => browse_the_web_1.BrowseTheWeb.as(actor).sleep(this.duration.toMillis());
    }
}
class IsVisible {
    constructor() {
        this.check = (thing) => protractor_1.protractor.ExpectedConditions.visibilityOf(thing);
        this.name = () => 'visible';
    }
}
class IsInvisible {
    constructor() {
        this.check = (thing) => protractor_1.protractor.ExpectedConditions.invisibilityOf(thing);
        this.name = () => 'invisible';
    }
}
class IsPresent {
    constructor() {
        this.check = (thing) => protractor_1.protractor.ExpectedConditions.presenceOf(thing);
        this.name = () => 'invisible';
    }
}
class Absent {
    constructor() {
        this.check = (thing) => protractor_1.protractor.ExpectedConditions.stalenessOf(thing);
        this.name = () => 'absent';
    }
}
class IsSelected {
    constructor() {
        this.check = (thing) => protractor_1.protractor.ExpectedConditions.elementToBeSelected(thing);
        this.name = () => 'selected';
    }
}
class IsClickable {
    constructor() {
        this.check = (thing) => protractor_1.protractor.ExpectedConditions.elementToBeClickable(thing);
        this.name = () => 'clickable';
    }
}
class WaitUntil {
    constructor(target, condition, timeout) {
        this.target = target;
        this.condition = condition;
        this.timeout = timeout;
    }
    performAs(actor) {
        return browse_the_web_1.BrowseTheWeb.as(actor).wait(this.condition.check(browse_the_web_1.BrowseTheWeb.as(actor).locate(this.target)), this.timeout.toMillis(), `${this.uppercased(this.target)} did not become ${this.condition.name()} within ${this.timeout}`);
    }
    uppercased(target) {
        let name = target.toString();
        return name[0].toUpperCase() + name.slice(1);
    }
}
/*
 [x] visibilityOf(elementFinder: ElementFinder): Function {
 [x] invisibilityOf(elementFinder: ElementFinder): Function {
 [x] presenceOf(elementFinder: ElementFinder): Function {
 [x] stalenessOf(elementFinder: ElementFinder): Function {
 [x] elementToBeSelected(elementFinder: ElementFinder): Function {
 [x] elementToBeClickable(elementFinder: ElementFinder): Function {

 [ ] alertIsPresent(): Function {
 [ ] textToBePresentInElement(elementFinder: ElementFinder, text: string): Function {
 [ ] textToBePresentInElementValue(elementFinder: ElementFinder, text: string): Function {
 [ ] titleContains(title: string): Function {
 [ ] titleIs(title: string): Function {
 [ ] urlContains(url: string): Function {
 [ ] urlIs(url: string): Function {
 */
//# sourceMappingURL=wait.js.map