"use strict";
class Target {
    constructor(name, locator) {
        this.name = name;
        this.locator = locator;
    }
    static the(name) {
        return new TargetBuilder(name);
    }
    of(...tokenReplacements) {
        return new Target(this.name, new Interpolated(this.locator).with(tokenReplacements));
    }
    called(newName) {
        return new Target(newName, this.locator);
    }
    resolveUsing(resolver) {
        return resolver(this.locator);
    }
    resolveAllUsing(resolver) {
        return resolver.all(this.locator);
    }
    toString() {
        return `the ${this.name}`;
    }
}
exports.Target = Target;
class Interpolated {
    constructor(locator) {
        this.locator = locator;
    }
    with(tokenReplacements) {
        // note: ProtractorBy is not compatible with WebdriverBy https://github.com/angular/protractor/issues/3508
        if (!this.canBeInterpolated()) {
            throw new Error(this.locator.toString() +
                ' is not a webdriver-compatible locator so you won\'t be able to use token replacement with it');
        }
        let cloned = Object.assign({}, this.locator);
        cloned.__proto__ = Object.getPrototypeOf(this.locator);
        cloned.value = this.interpolated(this.locator.value, tokenReplacements);
        return cloned;
    }
    canBeInterpolated() {
        return !!this.locator.value;
    }
    interpolated(template, replacements) {
        let argToken = /\\?\{(\d+)\\?\}/g, interpolator = (token, field) => replacements[field];
        return template.replace(argToken, interpolator);
    }
}
class TargetBuilder {
    constructor(name) {
        this.name = name;
    }
    located(byLocator) {
        return new Target(this.name, byLocator);
    }
    toString() {
        return `TargetBuilder for the ${this.name}`;
    }
}
exports.TargetBuilder = TargetBuilder;
//# sourceMappingURL=target.js.map