"use strict";
const domain_1 = require("../../serenity/domain");
const file_system_1 = require("../../serenity/io/file_system");
const serenity_1 = require("../../serenity/serenity");
const abilities_1 = require("../screenplay/abilities");
const photographer_timing_1 = require("./photographer-timing");
const md5_1 = require("ts-md5/dist/md5");
function photographer() {
    return Photographer.who(_ => _);
}
exports.photographer = photographer;
class ActivityOfInterest {
    constructor(resultsOfInterest) {
        this.resultsOfInterest = resultsOfInterest;
    }
    isAResultOfInterest(result) {
        return !!(result & this.resultsOfInterest);
    }
    isAnActivityOfInterest(activity) {
        return true;
    }
}
exports.ActivityOfInterest = ActivityOfInterest;
class PhotographySchedule {
    constructor() {
        this.Failures = new ActivityOfInterest(domain_1.Result.Failed);
        this.Tasks_and_Interactions = new ActivityOfInterest(domain_1.Result.Finished);
        this.Activity_Starts_and_Finishes = new photographer_timing_1.TimingBehaviour(new photographer_timing_1.TakeAPhoto(), new photographer_timing_1.TakeAPhoto());
        this.Activity_Starts = new photographer_timing_1.TimingBehaviour(new photographer_timing_1.TakeAPhoto(), new photographer_timing_1.NoPhoto());
        this.Activity_Finishes = new photographer_timing_1.TimingBehaviour(new photographer_timing_1.NoPhoto(), new photographer_timing_1.TakeAPhoto());
        this.activityOfInterest = this.Tasks_and_Interactions;
        this.photoTiming = this.Activity_Finishes;
        this.pathToPhotos = serenity_1.Default_Path_To_Reports;
        this.photoNamingStrategy = new Md5HashedPictureNames('png');
    }
    takesPhotosOf(activityOfInterest) {
        this.activityOfInterest = activityOfInterest;
        return this;
    }
    takesPhotosWhen(photoTiming) {
        this.photoTiming = photoTiming;
        return this;
    }
    storesPhotosAt(pathToPhotos) {
        this.pathToPhotos = pathToPhotos;
        return this;
    }
    build() {
        return new Photographer(this.activityOfInterest, this.photoTiming, new file_system_1.FileSystem(this.pathToPhotos), this.photoNamingStrategy);
    }
}
exports.PhotographySchedule = PhotographySchedule;
class Photographer {
    constructor(fieldOfInterest, behaviour, fs, naming = new Md5HashedPictureNames('png')) {
        this.fieldOfInterest = fieldOfInterest;
        this.fs = fs;
        this.naming = naming;
        this.strategy = new PhotoTakingStrategy(fieldOfInterest, behaviour);
    }
    static who(scheduler) {
        return scheduler(new PhotographySchedule()).build();
    }
    assignTo(stage) {
        this.stage = stage;
        this.stage.manager.registerInterestIn(Photographer.Events_of_Interest, this);
    }
    notifyOf(event) {
        switch (event.constructor.name) {
            case domain_1.ActivityStarts.name:
                this.strategy.activityStarts(event, this);
                break;
            case domain_1.ActivityFinished.name:
                this.strategy.activityFinished(event, this);
                break;
            default: break;
        }
    }
    canWork() {
        return !!this.stage && this.stage.theShowHasStarted();
    }
    photograph(subject, timestamp) {
        let promisedPicture = this.photographWorkOf(this.stage.theActorInTheSpotlight());
        this.stage.manager.notifyOf(new domain_1.PhotoAttempted(new domain_1.PhotoReceipt(subject, promisedPicture), timestamp));
    }
    photographWorkOf(actor) {
        let saveScreenshot = data => this.fs.store(this.naming.nameFor(data), new Buffer(data, 'base64'));
        let ignoreInactiveBrowserButReportAnyOther = (error) => {
            // todo: this needs further investigation; sometimes webdriver session dies before we can take a screenshot
            if (~error.message.indexOf('does not have a valid session ID') || ~error.message.indexOf('Session ID is null')) {
                // tslint:disable-next-line:no-console
                console.warn(`[Photographer] Looks like there was a problem with taking a photo of ${actor}: `, error.message);
                return undefined;
            }
            throw error;
        };
        return this.stage.manager.informOfWorkInProgress(abilities_1.BrowseTheWeb.as(actor).takeScreenshot()
            .then(saveScreenshot).then(path => new domain_1.Photo(path), error => ignoreInactiveBrowserButReportAnyOther(error)));
    }
}
Photographer.Events_of_Interest = [domain_1.ActivityStarts, domain_1.ActivityFinished];
exports.Photographer = Photographer;
class Md5HashedPictureNames {
    constructor(fileExtension = '') {
        this.fileExtension = fileExtension;
    }
    nameFor(base64encodedData) {
        return md5_1.Md5.hashStr(base64encodedData) + this.extension();
    }
    extension() {
        return !!this.fileExtension ? '.' + this.fileExtension : '';
    }
}
exports.Md5HashedPictureNames = Md5HashedPictureNames;
class PhotoTakingStrategy {
    constructor(interests, timing) {
        this.interests = interests;
        this.timing = timing;
    }
    activityStarts(event, photographer) {
        if (photographer.canWork() && this.interests.isAnActivityOfInterest(event.value)) {
            this.timing.takeABeforePhoto(event, photographer);
        }
    }
    activityFinished(event, photographer) {
        if (photographer.canWork() && this.interests.isAnActivityOfInterest(event.value.subject) && this.interests.isAResultOfInterest(event.value.result)) {
            this.timing.takeAnAfterPhoto(event, photographer);
        }
    }
}
//# sourceMappingURL=photographer.js.map