"use strict";
const fs = require("graceful-fs");
const mkdirp = require("mkdirp");
const path = require("path");
class FileSystem {
    constructor(root) {
        this.root = root;
    }
    store(relativePathToFile, data) {
        return this.ensureSpecified(relativePathToFile)
            .then(relativePath => this.prepareDirectory(relativePath))
            .then(absolutePath => this.write(absolutePath, data));
    }
    ensureSpecified(relativePathToFile) {
        return !!(relativePathToFile && relativePathToFile.length)
            ? Promise.resolve(relativePathToFile)
            : Promise.reject(new Error('Please specify where the file should be saved'));
    }
    prepareDirectory(relativePathToFile) {
        let absolutePath = path.resolve(this.root, relativePathToFile), parent = path.dirname(absolutePath);
        return new Promise((resolve, reject) => {
            mkdirp(parent, (error, dir) => {
                if (error) {
                    reject(error);
                }
                resolve(absolutePath);
            });
        });
    }
    write(path, data) {
        return new Promise((resolve, reject) => {
            fs.writeFile(path, data, error => {
                if (error) {
                    reject(error);
                }
                resolve(path);
            });
        });
    }
}
exports.FileSystem = FileSystem;
//# sourceMappingURL=file_system.js.map