"use strict";
const domain_1 = require("../domain");
const serenity_1 = require("../serenity");
const step_description_1 = require("./step_description");
const _ = require("lodash");
// todo: add Significance to the @step
class Step {
    constructor(stageManager) {
        this.stageManager = stageManager;
    }
    describedUsing(template) {
        let description = new step_description_1.StepDescription(template), decorated = this;
        return (target, propertyKey, descriptor) => {
            let performAs = descriptor.value, decorator = _.cloneDeep(descriptor);
            decorator.value = function (...args) {
                let activity = description.interpolateWith(this, args);
                return Promise.resolve()
                    .then(() => decorated.beforeStep(activity))
                    .then(() => performAs.apply(this, args))
                    .then(() => decorated.afterStep(activity), e => decorated.onFailure(activity, e));
            };
            return decorator;
        };
    }
    beforeStep(activity) {
        this.stageManager.notifyOf(new domain_1.ActivityStarts(activity));
    }
    afterStep(activity) {
        this.stageManager.notifyOf(new domain_1.ActivityFinished(new domain_1.Outcome(activity, domain_1.Result.SUCCESS)));
    }
    onFailure(activity, error) {
        this.stageManager.notifyOf(new domain_1.ActivityFinished(new domain_1.Outcome(activity, this.resultFrom(error), error)));
        return Promise.reject(error);
    }
    resultFrom(error) {
        const constructorOf = e => e && e.constructor ? e.constructor.name : '';
        // todo: sniff the exception to find out about the Result. Did the test fail, or was it compromised?
        return /AssertionError/.test(constructorOf(error))
            ? domain_1.Result.FAILURE
            : domain_1.Result.ERROR;
    }
}
exports.Step = Step;
function step(stepDescriptionTemplate) {
    return new Step(serenity_1.Serenity.stageManager()).describedUsing(stepDescriptionTemplate);
}
exports.step = step;
//# sourceMappingURL=step_annotation.js.map