"use strict";
const model_1 = require("../domain/model");
class StepDescription {
    constructor(template) {
        this.template = template;
        this.argToken = /{(\d+)}/g;
        this.fieldToken = /#(\w+)/g;
    }
    static from(template) {
        return new StepDescription(template);
    }
    interpolateWith(performable, argumentsOfThePerformAsMethod) {
        let name = this.determineStepName(this.template, performable, argumentsOfThePerformAsMethod);
        return new model_1.Activity(name);
    }
    using(source) {
        return (token, field) => {
            switch ({}.toString.call(source[field])) {
                case '[object Function]': return source[field]();
                case '[object Array]': return source[field].join(', ');
                case '[object Object]': return source[field].toString();
                case '[object Undefined]': return token;
                default: return source[field];
            }
        };
    }
    determineStepName(template, performable, args) {
        return template.
            replace(this.fieldToken, this.using(performable)).
            replace(this.argToken, this.using(args));
    }
}
exports.StepDescription = StepDescription;
//# sourceMappingURL=step_description.js.map