"use strict";
class Actor {
    constructor(name) {
        this.name = name;
        this.abilities = {};
    }
    static named(name) {
        return new Actor(name);
    }
    whoCan(...abilities) {
        abilities.forEach(ability => {
            this.abilities[ability.constructor.name] = ability;
        });
        return this;
    }
    abilityTo(doSomething) {
        if (!this.can(doSomething)) {
            throw new Error(`I don't have the ability to ${this.nameOf(doSomething)}, said ${this} sadly.`);
        }
        return this.abilities[this.nameOf(doSomething)];
    }
    attemptsTo(...tasks) {
        return tasks.reduce((previous, current) => {
            return previous.then(() => current.performAs(this));
        }, Promise.resolve(null));
    }
    toSee(question) {
        return question.answeredBy(this);
    }
    toString() {
        return this.name;
    }
    can(doSomething) {
        return !!this.abilities[this.nameOf(doSomething)];
    }
    nameOf(ability) {
        return ability.name;
    }
}
exports.Actor = Actor;
//# sourceMappingURL=actor.js.map